import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.awt.geom.*;

public class PetitPeintre
   extends JApplet implements ActionListener
{
   private static final int
      NB_COORDONNES = 1024;
   private static final Color []
      COULEURS =
   {
      Color.red, Color.green, Color.blue
   };
   private static final String []
      NOMS_COULEURS =
   {
      "Rouge", "Vert", "Bleu"
   };

   private int m_NbCoordonnes,
               m_DernireCoordonne;
   private int [] m_X;
   private int [] m_Y;
   private Color [] m_Couleur;

   //
   // -------------
   //
   private Color m_CouleurCourante;
   private void SetCouleurCourante(final Color c)
   {
      m_CouleurCourante = c;
   }
   private Color GetCouleurCourante()
   {
      return m_CouleurCourante;
   }


   //
   // -------------
   //
   private JCheckBoxMenuItem m_mnuCrayonDpos;
   private JRadioButtonMenuItem [] m_mnuCouleur;
   private JPopupMenu m_mnuPrincipal;

   //
   // -------------
   //
   private boolean m_CrayonDpos;
   private void SetCrayonDpos(final boolean Dpos)
   {
      m_CrayonDpos = Dpos;
      m_mnuCrayonDpos.setState (Dpos);
      MettreJourtat ();
   }
   private boolean GetCrayonDpos ()
   {
      return m_CrayonDpos;
   }


   //
   // -------------
   // Classe interne
   // -------------
   //
   private class PanneauDessin extends JPanel
   {
      public PanneauDessin ()
      {
         super ();
         setBackground (Color.black);
      }

      public void paint (Graphics g)
      {
         super.paint (g);
         Graphics2D g2D = (Graphics2D) g;
         if (m_NbCoordonnes > 0)
         {
            int vieuxX = m_X[0],
                vieuxY = m_Y[0];

            for (int i = 1; i < m_NbCoordonnes; i++)
            {
               g2D.setColor (m_Couleur[i]);
               g2D.draw (new Line2D.Float ( (float)vieuxX,
                                            (float)vieuxY,
                                            (float)m_X[i],
                                            (float)m_Y[i]));
               vieuxX = m_X[i];
               vieuxY = m_Y[i];
            }
         }
      }
   }
   //
   // -------------
   //
   private PanneauDessin m_Panneau;

   //
   // -------------
   //
   private JLabel m_Barretat;
   private void MettreJourtat ()
   {
      if (GetCrayonDpos ())
         m_Barretat.setText ("Le crayon est dpos; vous pouvez dessiner. Utilisez le menu contextuel pour voir les options disponibles");
      else
         m_Barretat.setText ("Le crayon est lev; vous ne pouvez pas dessiner. Utilisez le menu contextuel pour voir les options disponibles");
   }


   private void AjouterCoordonnes (final int x, final int y)
   {
      m_X[m_DernireCoordonne] = x;
      m_Y[m_DernireCoordonne] = y;
      m_Couleur[m_DernireCoordonne] = m_CouleurCourante;
      m_DernireCoordonne = (m_DernireCoordonne + 1) % NB_COORDONNES;
      if (m_NbCoordonnes < NB_COORDONNES)
         m_NbCoordonnes++;
   }


   public PetitPeintre ()
   {
      super ();
      m_X = new int [NB_COORDONNES];
      m_Y = new int [NB_COORDONNES];
      m_Couleur = new Color [NB_COORDONNES];
      m_NbCoordonnes = 0;
      m_DernireCoordonne = 0;
      SetCouleurCourante (COULEURS[0]);
   }

   public void init ()
   {
      boolean tatInitialCrayon = false;

      m_Barretat = new JLabel ("");
      getContentPane ().add (m_Barretat, BorderLayout.SOUTH);
      m_Panneau = new PanneauDessin ();
      getContentPane ().add (m_Panneau , BorderLayout.CENTER);
      m_Panneau.setBackground (Color.black);

      m_mnuPrincipal = new JPopupMenu ("Options");

      m_mnuCrayonDpos = new JCheckBoxMenuItem ("Crayon dpos", tatInitialCrayon);
      m_mnuPrincipal.add (m_mnuCrayonDpos);
      m_mnuCrayonDpos.addActionListener (this);

      SetCrayonDpos(tatInitialCrayon);

      m_mnuPrincipal.addSeparator ();

      ButtonGroup GroupeCouleur = new ButtonGroup ();

      m_mnuCouleur = new JRadioButtonMenuItem [COULEURS.length];
      for (int i = 0; i < m_mnuCouleur.length; i++)
      {
         // Only the zero-th item is set initially
         m_mnuCouleur[i] = new JRadioButtonMenuItem (NOMS_COULEURS[i], (i == 0));
         m_mnuPrincipal.add (m_mnuCouleur[i]);
         m_mnuCouleur[i].addActionListener (this);
         GroupeCouleur.add (m_mnuCouleur[i]);
      }

      addMouseMotionListener
      (
         new MouseMotionAdapter ()
         {
            public void mouseDragged (MouseEvent e)
            {
               if (GetCrayonDpos ())
               {
                  AjouterCoordonnes (e.getX (), e.getY ());
                  repaint ();
               }
            }
         }
      );

      addMouseListener
      (
         new MouseAdapter ()
         {
            public void mousePressed (MouseEvent e)
            {
               MontrerPopup (e);
            }
            public void mouseReleased (MouseEvent e)
            {
               MontrerPopup (e);
            }
            private boolean MontrerPopup (MouseEvent e)
            {
               boolean Fait = false;
               if (e.isPopupTrigger ())
               {
                  m_mnuPrincipal.show
                     (e.getComponent (), e.getX (), e.getY ());
                  Fait = true;
               }
               return Fait;

            }
         }
      );
   }

   public void actionPerformed (ActionEvent e)
   {
      if (e.getSource () == m_mnuCrayonDpos)
      {
         SetCrayonDpos (!GetCrayonDpos ());
      }
      else // It's a color change
      {
         for (int i = 0; i < m_mnuCouleur.length; i++)
         {
            if (e.getSource () == m_mnuCouleur[i])
            {
               SetCouleurCourante (COULEURS[i]);
            }
         }
      }
   }
}
